@@ -40,6 +40,7 @@
 #include "hildon-file-selection.h"
 #include "hildon-file-chooser-dialog.h"
 #include "hildon-file-system-private.h"
+#include "hildon-file-system-settings.h"
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkbox.h>
 #include <gtk/gtkentry.h>
@@ -55,9 +56,11 @@
 #include <hildon/hildon-defines.h>
 #include <hildon/hildon-banner.h>
 #include <hildon/hildon-helper.h>
+#include <hildon/hildon-note.h>
 #include <libintl.h>
 #include <gdk/gdkx.h>
 #include <stdlib.h>
+#include <glib.h>
 
 #include "hildon-file-common-private.h"
 #define HILDON_RESPONSE_FOLDER_BUTTON 12345
@@ -282,6 +285,39 @@ get_path_length_from_uri(const char *uri)
   return len;
 }
 
+static gint
+get_global_pane_position ()
+{
+  GError *error = NULL;
+  GKeyFile *keys;
+  gint pos;
+
+  keys = hildon_file_system_open_user_settings ();
+  pos = g_key_file_get_integer (keys, "default", "pane_position", &error);
+  if (error)
+    {
+      pos = 250;
+      if (!g_error_matches (error, G_KEY_FILE_ERROR,
+			    G_KEY_FILE_ERROR_KEY_NOT_FOUND)
+	  && !g_error_matches (error, G_KEY_FILE_ERROR,
+			       G_KEY_FILE_ERROR_GROUP_NOT_FOUND))
+	fprintf (stderr, "%s\n", error->message);
+      g_error_free (error);
+    }
+
+  g_key_file_free (keys);
+  return pos;
+}
+
+static void
+set_global_pane_position (gint pos)
+{
+  GKeyFile *keys = hildon_file_system_open_user_settings ();
+  g_key_file_set_integer (keys, "default", "pane_position", pos);
+  hildon_file_system_write_user_settings (keys);
+  g_key_file_free (keys);
+}
+
 static void
 hildon_file_chooser_dialog_set_limit(HildonFileChooserDialog *self)
 {
@@ -1120,12 +1156,18 @@ static void create_folder_callback(GtkFileSystemHandle *handle,
 
     if (error) {
         if (g_error_matches(error, GTK_FILE_SYSTEM_ERROR,
-              GTK_FILE_SYSTEM_ERROR_ALREADY_EXISTS))
+			    GTK_FILE_SYSTEM_ERROR_ALREADY_EXISTS))
             message = _("ckdg_ib_folder_already_exists");
         else
-            message = _("sfil_ni_operation_failed");
+            message = HCS("sfil_ni_operation_failed");
+
+	fprintf (stderr, "domain %d (%d)\n",
+		 error->domain, GTK_FILE_SYSTEM_ERROR);
+	fprintf (stderr, "code %d (%d)\n",
+		 error->code, GTK_FILE_SYSTEM_ERROR_ALREADY_EXISTS);
+
+        fprintf (stderr, "%s\n", error->message);
 
-        ULOG_ERR(error->message);
         hildon_banner_show_information (GTK_WIDGET(self), NULL, message);
         hildon_file_chooser_dialog_select_text(self->priv);
         gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_OK, TRUE);
@@ -1240,7 +1282,8 @@ static gboolean hildon_file_chooser_dialog_location_pressed(GtkWidget
 {
   HildonFileChooserDialog *self = HILDON_FILE_CHOOSER_DIALOG(data);
 
-  if (self->priv->action == GTK_FILE_CHOOSER_ACTION_SAVE)
+  if (self->priv->action == GTK_FILE_CHOOSER_ACTION_SAVE ||
+	self->priv->action == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER)
     handle_folder_popup(self);
 
   return FALSE;
@@ -1304,10 +1347,13 @@ static void response_handler(GtkWidget * widget, gint arg1, gpointer data)
 
                     if (path_length > self->priv->max_full_path_length)
                     {
+                        GtkWidget *w;
                         g_signal_stop_emission_by_name(widget, "response");
                         hildon_file_chooser_dialog_select_text(priv);
-                        hildon_banner_show_information
-                          (widget, NULL, HCS("file_ib_name_too_long"));
+                        w = hildon_note_new_information(GTK_WINDOW(widget),
+                                FM("sfil_ni_target_foldername_long"));
+                        gtk_dialog_run(GTK_DIALOG(w));
+                        gtk_widget_destroy(w);
                     }
                     g_free(uri);
                 }
@@ -1561,10 +1607,21 @@ static void hildon_file_chooser_dialog_show(GtkWidget * widget)
 static void hildon_file_chooser_dialog_destroy(GtkObject * obj)
 {
     HildonFileChooserDialog *dialog;
+    gint pos;
 
     ULOG_DEBUG_F("entered");
 
     dialog = HILDON_FILE_CHOOSER_DIALOG(obj);
+    
+    if (dialog->priv->filetree)
+      {
+	pos = -1;
+	g_object_get (dialog->priv->filetree,
+		      "pane-position", &pos,
+		      NULL);
+	if (pos >= 0)
+	  set_global_pane_position (pos);
+      }
 
     /* We need sometimes to break cyclic references */
 
@@ -1578,6 +1635,8 @@ static void hildon_file_chooser_dialog_destroy(GtkObject * obj)
         dialog->priv->model = NULL;
     }
 
+    dialog->priv->filetree = NULL;
+
     GTK_OBJECT_CLASS(hildon_file_chooser_dialog_parent_class)->
         destroy(obj);
 }
@@ -1951,9 +2010,12 @@ hildon_file_chooser_dialog_constructor(GType type,
     priv = HILDON_FILE_CHOOSER_DIALOG(obj)->priv;
 
     g_assert(priv->model);
-    priv->filetree = g_object_new(HILDON_TYPE_FILE_SELECTION,
-      "model", priv->model, "visible-columns",
-      HILDON_FILE_SELECTION_SHOW_NAME | HILDON_FILE_SELECTION_SHOW_MODIFIED,
+    priv->filetree = g_object_new
+      (HILDON_TYPE_FILE_SELECTION,
+       "model", priv->model,
+       "visible-columns", (HILDON_FILE_SELECTION_SHOW_NAME
+			   | HILDON_FILE_SELECTION_SHOW_MODIFIED),
+       "pane-position", get_global_pane_position (),
       NULL);
 
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(obj)->vbox),
